/**********************************************************************************
 * $URL: https://source.sakaiproject.org/svn/providers/tags/sakai_2-4-0/cm/cm-authz-provider/src/java/org/sakaiproject/coursemanagement/impl/provider/CourseOfferingRoleResolver.java $
 * $Id: CourseOfferingRoleResolver.java 20453 2007-01-19 00:26:56Z jholtzman@berkeley.edu $
 ***********************************************************************************
 *
 * Copyright (c) 2006 The Sakai Foundation.
 * 
 * Licensed under the Educational Community License, Version 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 *      http://www.opensource.org/licenses/ecl1.php
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 **********************************************************************************/
package org.sakaiproject.coursemanagement.impl.provider;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.sakaiproject.coursemanagement.api.CourseManagementService;
import org.sakaiproject.coursemanagement.api.CourseOffering;
import org.sakaiproject.coursemanagement.api.Membership;
import org.sakaiproject.coursemanagement.api.Section;

/**
 * Resolves user roles in CourseOfferings.
 * 
 * @author <a href="mailto:jholtzman@berkeley.edu">Josh Holtzman</a>
 *
 */
public class CourseOfferingRoleResolver implements RoleResolver {
	private static final Log log = LogFactory.getLog(CourseOfferingRoleResolver.class);
	
	/** Map of CM course offering roles to Sakai roles */
	Map roleMap;

	/**
	 * {@inheritDoc}
	 */
	public Map<String, String> getUserRoles(CourseManagementService cmService, Section section) {
		Map<String, String> userRoleMap = new HashMap<String, String>();
		String coEid = section.getCourseOfferingEid();
		
		// Get the members of this course offering
		Set<Membership> coMembers = cmService.getCourseOfferingMemberships(coEid);
		Set<Membership> equivMembers = new HashSet<Membership>();
		
		// Get the members of equivalent course offerings, and add them to the set of equivalent memberships
		Set<CourseOffering> equivalentCourseOfferings = cmService.getEquivalentCourseOfferings(coEid);
		for(Iterator<CourseOffering> iter = equivalentCourseOfferings.iterator(); iter.hasNext();) {
			CourseOffering equivCo = iter.next();
			equivMembers.addAll(cmService.getCourseOfferingMemberships(equivCo.getEid()));
		}

		// Add the course offering memebers
		addMemberRoles(userRoleMap, coMembers);
		
		// Add the equivalent course offering members (but don't override any roles in the original course offering)
		addMemberRoles(userRoleMap, equivMembers);
		
		if(coMembers != null) {
		}
		return userRoleMap;
	}

	private void addMemberRoles(Map<String, String> userRoleMap, Set<Membership>coMembers) {
		for(Iterator<Membership> iter = coMembers.iterator(); iter.hasNext();) {
			Membership membership = iter.next();
			if(userRoleMap.containsKey(membership.getUserId())) {
				// Don't override existing roles in the map.
			} else {
				String sakaiRole = convertRole(membership.getRole());
				if(sakaiRole != null) {
					userRoleMap.put(membership.getUserId(), sakaiRole);
				}
			}
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Map<String, String> getGroupRoles(CourseManagementService cmService, String userEid) {
		Map<String, String> sectionRoles = new HashMap<String, String>();
		
		// Find all of the course offerings for which this user is a member
		Map<String, String> courseOfferingRoles = cmService.findCourseOfferingRoles(userEid);
		if(log.isDebugEnabled()) log.debug("Found " + courseOfferingRoles.size() + " course offering roles for " + userEid);

		// Add all of the equivalent course offerings-> role mappings for this user
		Set<String> coEids = new HashSet(courseOfferingRoles.keySet());
		for(Iterator<String> coIter = coEids.iterator(); coIter.hasNext();) {
			String coEid = coIter.next();
			Set<CourseOffering> equivOfferings = cmService.getEquivalentCourseOfferings(coEid);
			for(Iterator<CourseOffering> equivIter = equivOfferings.iterator(); equivIter.hasNext();) {
				CourseOffering equiv = equivIter.next();
				// Use the role in the original course offering for this equivalent course offering
				courseOfferingRoles.put(equiv.getEid(), courseOfferingRoles.get(coEid));
			}
		}
		
		for(Iterator<String> coIter = courseOfferingRoles.keySet().iterator(); coIter.hasNext();) {
			String coEid = coIter.next();
			String coRole = courseOfferingRoles.get(coEid);
			
			// If this role shouldn't be part of the site, ignore the membership in this course offering
			String sakaiRole = convertRole(coRole);
			if(sakaiRole == null) {
				if(log.isDebugEnabled()) log.debug("Course offering role " + coRole + " is not mapped to a sakai role.  Skipping this membership.");
				continue;
			}
			
			if(log.isDebugEnabled()) log.debug(userEid + " has role=" + coRole + " in course offering " + coEid);
			// Get the sections in each course offering
			Set<Section> sections = cmService.getSections(coEid);
			for(Iterator<Section> secIter = sections.iterator(); secIter.hasNext();) {
				// Add the section EIDs and the converted *CourseOffering* role to the sectionRoles map
				Section section = secIter.next();
				sectionRoles.put(section.getEid(), sakaiRole);
			}
		}
		return sectionRoles;
	}


	public String convertRole(String cmRole) {
		if (cmRole == null) {
			log.warn("Can not convert CM role 'null' to a sakai role.");
			return null;
		}
		String sakaiRole = (String)roleMap.get(cmRole);
		if(sakaiRole== null) {
			log.warn("Unable to find sakai role for CM role " + cmRole);
			return null;
		} else {
			return sakaiRole;
		}
	}
	// Dependency injection
	
	public void setRoleMap(Map roleMap) {
		this.roleMap = roleMap;
	}

}
